/******************************************************************************\
|* Copyright 2010, Dream Chip Technologies GmbH. used with permission by      *|
|* VeriSilicon.                                                               *|
|* Copyright (c) <2020> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets of    *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */


/**
 * @file cameric_reg_description.c
 *
 * @brief
 *   ADD_DESCRIPTION_HERE
 *
 *****************************************************************************/
#include <ebase/types.h>
#include <ebase/trace.h>
#include <ebase/builtins.h>

#include <common/return_codes.h>
#include <common/array_size.h>

#include <hal/hal_api.h>

#include "mrv_all_bits.h"

#include "cameric_reg_description.h"



/******************************************************************************
 * local macro definitions
 *****************************************************************************/
CREATE_TRACER( CAMERIC_REG_DESCRIPTION_INFO  , "CAMERIC-REG-DESCRIPTION: ", INFO     , 0 );
CREATE_TRACER( CAMERIC_REG_DESCRIPTION_WARN  , "CAMERIC-REG-DESCRIPTION: ", WARNING  , 1 );
CREATE_TRACER( CAMERIC_REG_DESCRIPTION_ERROR , "CAMERIC-REG-DESCRIPTION: ", ERROR    , 1 );



/******************************************************************************
 * local type definitions
 *****************************************************************************/


/* include register descriptions which is generated by sig-flow */
#include "cameric_reg_description_main_control.h"
#include "cameric_reg_description_img_effects.h"
#include "cameric_reg_description_super_impose.h"
#include "cameric_reg_description_isp_main.h"
#include "cameric_reg_description_isp_flash.h"
#include "cameric_reg_description_isp_shutter.h"
#include "cameric_reg_description_cproc.h"
#include "cameric_reg_description_mrsz.h"
#include "cameric_reg_description_srsz.h"
#include "cameric_reg_description_mi.h"
#include "cameric_reg_description_jpe.h"
#include "cameric_reg_description_smia.h"
#include "cameric_reg_description_mipi.h"
#include "cameric_reg_description_afm.h"
#include "cameric_reg_description_lsc.h"
#include "cameric_reg_description_is.h"
#include "cameric_reg_description_hist.h"
#include "cameric_reg_description_filt.h"
#include "cameric_reg_description_cac.h"
#include "cameric_reg_description_cnr.h"
#include "cameric_reg_description_exp.h"
#include "cameric_reg_description_bls.h"
#include "cameric_reg_description_dpf.h"
#include "cameric_reg_description_dpcc.h"
#include "cameric_reg_description_wdr.h"
#include "cameric_reg_description_awb.h"
#include "cameric_reg_description_vsm.h"



/******************************************************************************
 * local variable declarations
 *****************************************************************************/



/******************************************************************************
 * local function prototypes
 *****************************************************************************/



/******************************************************************************
 * local functions
 *****************************************************************************/



/*****************************************************************************/
/**
 *          CamerIcInitMainControlRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitMainControlRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(MainControlRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->vi_ccl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->vi_id));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->vi_iccl));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->vi_ircl));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->vi_dpcl));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        MainControlRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitImgEffectsRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitImgEffectsRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(ImgEffectsRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_color_sel));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_mat_1));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_mat_2));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_mat_3));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_mat_4));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_mat_5));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_tint));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_ctrl_shd));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->img_eff_sharpen));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        ImgEffectsRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitSuperImposeRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the SuperImpose-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitSuperImposeRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(SuperImposeRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->super_imp_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->super_imp_offset_x));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->super_imp_offset_y));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->super_imp_color_y));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->super_imp_color_cb));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->super_imp_color_cr));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        SuperImposeRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitIspMainRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the IspMain-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitIspMainRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(IspMainRegisterMap); i++ )
    {
        switch ( ( ((i>=9) && (i<=25)) ? 9 :
                    ((i>=26) && (i<=42)) ? 26 :
                    ((i>=43) && (i<=59)) ? 43 :
                    ((i>=96) && (i<=104)) ? 96 :
                    ((i>=106) && (i<=122)) ? 106 : i ) )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_acq_prop));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_acq_h_offs));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_acq_v_offs));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_acq_h_size));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_acq_v_size));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_acq_nr_frames));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_gamma_dx_lo));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_gamma_dx_hi));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->degamma_r_y_block_arr[i-9]));
                break;

            case 26:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->degamma_g_y_block_arr[i-26]));
                break;

            case 43:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->degamma_b_y_block_arr[i-43]));
                break;

            case 60:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_prop));
                break;

            case 61:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_h_offs));
                break;

            case 62:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_v_offs));
                break;

            case 63:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_h_size));
                break;

            case 64:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_v_size));
                break;

            case 65:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_frames));
                break;

            case 66:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_ref));
                break;

            case 67:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_thresh));
                break;

            case 68:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_gain_g));
                break;

            case 69:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_gain_rb));
                break;

            case 70:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_white_cnt));
                break;

            case 71:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_awb_mean));
                break;

            case 72:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_0));
                break;

            case 73:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_1));
                break;

            case 74:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_2));
                break;

            case 75:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_3));
                break;

            case 76:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_4));
                break;

            case 77:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_5));
                break;

            case 78:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_6));
                break;

            case 79:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_7));
                break;

            case 80:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cc_coeff_8));
                break;

            case 81:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_h_offs));
                break;

            case 82:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_v_offs));
                break;

            case 83:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_h_size));
                break;

            case 84:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_v_size));
                break;

            case 85:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_demosaic));
                break;

            case 86:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flags_shd));
                break;

            case 87:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_h_offs_shd));
                break;

            case 88:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_v_offs_shd));
                break;

            case 89:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_h_size_shd));
                break;

            case 90:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_out_v_size_shd));
                break;

            case 91:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_imsc));
                break;

            case 92:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_ris));
                break;

            case 93:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_mis));
                break;

            case 94:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_icr));
                break;

            case 95:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_isr));
                break;

            case 96:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->cross_talk_coef_block_arr[i-96]));
                break;

            case 105:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_gamma_out_mode));
                break;

            case 106:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->gamma_out_y_block_arr[i-106]));
                break;

            case 123:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_err));
                break;

            case 124:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_err_clr));
                break;

            case 125:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_frame_count));
                break;

            case 126:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_ct_offset_r));
                break;

            case 127:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_ct_offset_g));
                break;

            case 128:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_ct_offset_b));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        IspMainRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitIspFlashRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the IspFlash-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitIspFlashRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(IspFlashRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flash_cmd));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flash_config));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flash_prediv));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flash_delay));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flash_time));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_flash_maxp));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        IspFlashRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}


/*****************************************************************************/
/**
 *          CamerIcInitIspShutterRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the IspShutter-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitIspShutterRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(IspShutterRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_sh_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_sh_prediv));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_sh_delay));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_sh_time));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        IspShutterRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitCprocRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitCprocRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(CprocRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->cproc_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->cproc_contrast));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->cproc_brightness));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->cproc_saturation));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->cproc_hue));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        CprocRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitMrszRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitMrszRegisterMap
(
    const uint32_t BASE
)
{

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}


/*****************************************************************************/
/**
 *          CamerIcInitSrszRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitSrszRegisterMap
(
    const uint32_t BASE
)
{

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );


    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitMiRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MIPI-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitMiRegisterMap
(
    const uint32_t BASE
)
{

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );


    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitJpeRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MIPI-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitJpeRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(JpeRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_gen_header));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_encode));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_init));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_y_scale_en));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_cbcr_scale_en));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_table_flush));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_enc_hsize));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_enc_vsize));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_pic_format));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_restart_interval));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tq_y_select));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tq_u_select));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tq_v_select));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_dc_table_select));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_ac_table_select));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_table_data));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_table_id));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tac0_len));
                break;

            case 18:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tdc0_len));
                break;

            case 19:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tac1_len));
                break;

            case 20:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_tdc1_len));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_encoder_busy));
                break;

            case 22:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_header_mode));
                break;

            case 23:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_encode_mode));
                break;

            case 24:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_debug));
                break;

            case 25:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_error_imr));
                break;

            case 26:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_error_ris));
                break;

            case 27:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_error_mis));
                break;

            case 28:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_error_icr));
                break;

            case 29:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_error_isr));
                break;

            case 30:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_status_imr));
                break;

            case 31:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_status_ris));
                break;

            case 32:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_status_mis));
                break;

            case 33:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_status_icr));
                break;

            case 34:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->jpe_status_isr));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        JpeRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitSmiaRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MIPI-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitSmiaRegisterMap
(
    const uint32_t BASE
)
{
    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

#if (MARVIN_FEATURE_SMIA != MARVIN_FEATURE_EXIST_NOT)
    for ( i=0; i<ARRAY_SIZE(SmiaRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_status));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_imsc));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_ris));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_mis));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_icr));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_isr));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_data_format_sel));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_sof_emb_data_lines));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_emb_hstart));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_emb_hsize));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_emb_vstart));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_num_lines));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_emb_data_fifo));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->smia_fifo_fill_level));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        SmiaRegisterMap[i].Address = address;
    }
#endif /* (MARVIN_FEATURE_SMIA != MARVIN_FEATURE_EXIST_NOT) */

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitMipiRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MIPI-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitMipiRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(MipiRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_status));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_imsc));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_ris));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_mis));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_icr));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_isr));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_cur_data_id));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_img_data_sel));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_add_data_sel_1));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_add_data_sel_2));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_add_data_sel_3));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_add_data_sel_4));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_add_data_fifo));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_add_data_fill_level));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_compressed_mode));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_frame));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_gen_short_dt));
                break;

            case 18:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_gen_short_8_9));
                break;

            case 19:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_gen_short_a_b));
                break;

            case 20:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_gen_short_c_d));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->mipi_gen_short_e_f));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        MipiRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitAfmRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the LSC-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitAfmRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(AfmRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_lt_a));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_rb_a));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_lt_b));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_rb_b));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_lt_c));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_rb_c));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_thres));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_var_shift));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_sum_a));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_sum_b));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_sum_c));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_lum_a));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_lum_b));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_afm_lum_c));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        AfmRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitLscRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the LSC-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitLscRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(LscRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_r_table_addr));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_gr_table_addr));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_b_table_addr));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_r_table_data));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_gr_table_data));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_b_table_data));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_status));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xgrad_01));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xgrad_23));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xgrad_45));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xgrad_67));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ygrad_01));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ygrad_23));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ygrad_45));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ygrad_67));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xsize_01));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xsize_23));
                break;

            case 18:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xsize_45));
                break;

            case 19:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_xsize_67));
                break;

            case 20:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ysize_01));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ysize_23));
                break;

            case 22:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ysize_45));
                break;

            case 23:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_lsc_ysize_67));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        LscRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitIsRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Exposure-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitIsRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(IsRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_recenter));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_h_offs));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_v_offs));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_h_size));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_v_size));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_max_dx));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_max_dy));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_displace));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_h_offs_shd));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_v_offs_shd));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_h_size_shd));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_is_v_size_shd));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        IsRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitHistRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Exposure-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitHistRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(HistRegisterMap); i++ )
    {
        switch ( ((i>=5) && (i<=20)) ? 5 : i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_prop));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_h_offs));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_v_offs));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_h_size));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_v_size));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->histogram_measurement_result_arr[i-5]));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_00to30));
                break;

            case 22:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_40to21));
                break;

            case 23:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_31to12));
                break;

            case 24:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_22to03));
                break;

            case 25:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_13to43));
                break;

            case 26:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_04to34));
                break;

            case 27:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_hist_weight_44));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        HistRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitFiltRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Exposure-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitFiltRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(FiltRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_mode));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_thresh_bl0));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_thresh_bl1));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_thresh_sh0));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_thresh_sh1));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_lum_weight));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_fac_sh1));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_fac_sh0));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_fac_mid));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_fac_bl0));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_filt_fac_bl1));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        FiltRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitCacRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Exposure-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitCacRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(CacRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_count_start));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_a));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_b));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_c));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_x_norm));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cac_y_norm));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        CacRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}


/*****************************************************************************/
/**
 *          CamerIcInitCnrRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Chroma-Noise-Reduction
 *          -Registerblock relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitCnrRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(CnrRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cnr_linesize));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cnr_threshold_c1));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_cnr_threshold_c2));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        CnrRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitExpRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Exposure-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitExpRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(ExpRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_h_offset));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_v_offset));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_h_size));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_v_size));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_00));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_10));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_20));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_30));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_40));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_01));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_11));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_21));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_31));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_41));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_02));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_12));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_22));
                break;

            case 18:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_32));
                break;

            case 19:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_42));
                break;

            case 20:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_03));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_13));
                break;

            case 22:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_23));
                break;

            case 23:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_33));
                break;

            case 24:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_43));
                break;

            case 25:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_04));
                break;

            case 26:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_14));
                break;

            case 27:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_24));
                break;

            case 28:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_34));
                break;

            case 29:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_exp_mean_44));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        ExpRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitBlsRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the Bls-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitBlsRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(BlsRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_samples));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_h1_start));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_h1_stop));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_v1_start));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_v1_stop));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_h2_start));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_h2_stop));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_v2_start));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_v2_stop));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_a_fixed));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_b_fixed));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_c_fixed));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_d_fixed));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_a_measured));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_b_measured));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_c_measured));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_bls_d_measured));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        BlsRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitDpfRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitDpfRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(DpfRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_mode));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_strength_r));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_strength_g));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_strength_b));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_s_weight_g_1_4));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_s_weight_g_5_6));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_s_weight_rb_1_4));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_s_weight_rb_5_6));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[0]));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[1]));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[2]));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[3]));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[4]));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[5]));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[6]));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[7]));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[8]));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[9]));
                break;

            case 18:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[10]));
                break;

            case 19:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[11]));
                break;

            case 20:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[12]));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[13]));
                break;

            case 22:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[14]));
                break;

            case 23:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[15]));
                break;

            case 24:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->nlf_lookup_table_block_arr[16]));
                break;

            case 25:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_nf_gain_r));
                break;

            case 26:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_nf_gain_gr));
                break;

            case 27:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_nf_gain_gb));
                break;

            case 28:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpf_nf_gain_b));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        DpfRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitDpccRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the MainControl-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitDpccRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(DpccRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_mode));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_output_mode));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_set_use));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_methods_set_1));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_methods_set_2));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_methods_set_3));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_line_thresh_1));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_line_mad_fac_1));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_pg_fac_1));
                break;

            case 9:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rnd_thresh_1));
                break;

            case 10:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rg_fac_1));
                break;

            case 11:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_line_thresh_2));
                break;

            case 12:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_line_mad_fac_2));
                break;

            case 13:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_pg_fac_2));
                break;

            case 14:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rnd_thresh_2));
                break;

            case 15:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rg_fac_2));
                break;

            case 16:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_line_thresh_3));
                break;

            case 17:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_line_mad_fac_3));
                break;

            case 18:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_pg_fac_3));
                break;

            case 19:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rnd_thresh_3));
                break;

            case 20:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rg_fac_3));
                break;

            case 21:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_ro_limits));
                break;

            case 22:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_rnd_offs));
                break;

            case 23:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_bpt_ctrl));
                break;

            case 24:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_bpt_number));
                break;

            case 25:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_bpt_addr));
                break;

            case 26:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_dpcc_bpt_data));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        DpccRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitWdrRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the WDR-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitWdrRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(WdrRegisterMap); i++ )
    {
        switch ( ((i>=5) && (i<=37)) ? 5 : ((i>=44) && (i<=76)) ? 44 :  i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_ctrl));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_1));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_2));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_3));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_4));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->wdr_tone_mapping_curve_y_block_arr[(i-5)]));
                break;

            case 38:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_offset));
                break;

            case 39:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_deltamin));
                break;

            case 40:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_1_shd));
                break;

            case 41:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_2_shd));
                break;

            case 42:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_3_shd));
                break;

            case 43:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_wdr_tonecurve_4_shd));
                break;

            case 44:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->wdr_tone_mapping_curve_y_shd_block_arr[i-44]));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        WdrRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitAwbRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the AWB-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitAwbRegisterMap
(
    const uint32_t BASE
)
{

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/*****************************************************************************/
/**
 *          CamerIcInitAwbRegisterMap()
 *
 * @brief   Initializes the Register-Addresses of the VSM-Registerblock
 *          relating to the Base-Address
 *
 * @note    This function will be moved to the Sig-Flow ( PLEASE DON'T EDIT )
 *
 * @return  Return the result of the function call.
 *
 *****************************************************************************/
static RESULT CamerIcInitVsmRegisterMap
(
    const uint32_t BASE
)
{
    volatile MrvAllRegister_t *ptCamerIcRegMap = (MrvAllRegister_t *)(uintptr_t)BASE;

    uint32_t address;
    uint32_t i;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    for ( i=0; i<ARRAY_SIZE(VsmRegisterMap); i++ )
    {
        switch ( i )
        {
            case 0:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_mode));
                break;

            case 1:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_h_offs));
                break;

            case 2:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_v_offs));
                break;

            case 3:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_h_size));
                break;

            case 4:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_v_size));
                break;

            case 5:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_h_segments));
                break;

            case 6:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_v_segments));
                break;

            case 7:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_delta_h));
                break;

            case 8:
                address = ((uint32_t)(uintptr_t)(&ptCamerIcRegMap->isp_vsm_delta_v));
                break;

            default:
                address = 0xFFFFFFFFU;
                break;
        }

        VsmRegisterMap[i].Address = address;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}



/******************************************************************************
 * Implementation of CamerIc Register Driver API Functions
 *****************************************************************************/

/******************************************************************************
 * CamerIcInitRegisterDescription()
 *****************************************************************************/
RESULT CamerIcInitRegDescriptionDrv
(
    const CamerIcRegDescriptionDrvConfig_t  *pConfig,
    CamerIcDrvContext_t *regDesctx
)
{
    RESULT result = RET_SUCCESS;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    //gDrvCtx = (CamerIcDrvContext_t *)malloc( sizeof(CamerIcDrvContext_t) );
    if ( regDesctx == NULL )
    {
        TRACE( CAMERIC_REG_DESCRIPTION_ERROR,  "%s: Can't allocate CamerIC context\n",  __func__ );
        return ( RET_OUTOFMEM );
    }
    MEMSET( regDesctx, 0, sizeof(CamerIcDrvContext_t) );

    if ( pConfig->HalHandle )
    {
        /* setup CamerIc software HAL */
        result = HalAddRef( pConfig->HalHandle );
        if ( result != RET_SUCCESS )
        {
            TRACE( CAMERIC_REG_DESCRIPTION_ERROR, "%s: Can't reference HAL (%d)\n",  __func__ );
            return ( result );
        }
        regDesctx->HalHandle = pConfig->HalHandle;
    }

    result = CamerIcInitMainControlRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitImgEffectsRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitSuperImposeRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitIspMainRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitIspFlashRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitIspShutterRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitCprocRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitMrszRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitSrszRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitMiRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitJpeRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitSmiaRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitMipiRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitAfmRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitLscRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitIsRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitHistRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitFiltRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitCacRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitCnrRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitExpRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitBlsRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitDpfRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitDpccRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitWdrRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitAwbRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    result = CamerIcInitVsmRegisterMap( HAL_BASEADDR_MARVIN );
    if ( result != RET_SUCCESS )
    {
        return ( result );
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( result );
}



/******************************************************************************
 * CamerIcInitRegisterDescription()
 *****************************************************************************/
RESULT CamerIcReleaseRegDescriptionDrv
(
    CamerIcDrvContext_t *regDesctx
)
{
    RESULT result = RET_SUCCESS;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    if ( regDesctx->HalHandle )
    {
        result = HalDelRef( regDesctx->HalHandle );
        if ( result != RET_SUCCESS )
        {
            TRACE( CAMERIC_REG_DESCRIPTION_ERROR, "%s: Can't dereference HAL (%d)\n",  __func__ );
            return ( result );
        }
        regDesctx->HalHandle = NULL;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( result );
}



/******************************************************************************
 * CamerIcGetRegister()
 *****************************************************************************/
RESULT CamerIcGetRegister
(
    CamerIcDrvContext_t *regDesctx,
    const uint32_t  address,
    uint32_t        *value
)
{
    RESULT result = RET_SUCCESS;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );
    if(regDesctx == NULL)
    {
        TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: NUll pointer\n", __func__ );
        return ( RET_NULL_POINTER );
    }


    if ( value == NULL )
    {
        return ( RET_NULL_POINTER );
    }

    if ( regDesctx->HalHandle )
    {
        *value = HalReadReg( regDesctx->HalHandle, address );
    }
    else
    {
        *value = 0U;
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( result );
}



/******************************************************************************
 * CamerIcSetRegister()
 *****************************************************************************/
RESULT CamerIcSetRegister
(
    CamerIcDrvContext_t *regDesctx,
    const uint32_t  address,
    const uint32_t  value
)
{
    RESULT result = RET_SUCCESS;

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    if ( regDesctx->HalHandle )
    {
        HalWriteReg( regDesctx->HalHandle, address, value );
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( result );
}



/******************************************************************************
 * CamerIcGetRegisterDescription()
 *****************************************************************************/
RESULT CamerIcGetRegisterDescription
(
    const CamerIcModuleId_t ModuleId,
    uint32_t                *NumRegisters,
    RegDescription_t        **Registers
)
{
    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (enter)\n", __func__ );

    if ( NumRegisters == NULL )
    {
        TRACE( CAMERIC_REG_DESCRIPTION_WARN, "%s: parameter 'NumRegisters' is invalid\n", __func__ );
        return ( RET_NULL_POINTER );
    }

    if ( Registers == NULL )
    {
        TRACE( CAMERIC_REG_DESCRIPTION_WARN, "%s: parameter 'Registers' is invalid\n", __func__ );
        return ( RET_NULL_POINTER );
    }

    switch ( ModuleId )
    {
        case CAMERIC_MODULE_MAIN_CONTROL:
            *NumRegisters = ARRAY_SIZE( MainControlRegisterMap );
            *Registers = MainControlRegisterMap;
            break;

        case CAMERIC_MODULE_IMG_EFFECTS:
            *NumRegisters = ARRAY_SIZE( ImgEffectsRegisterMap );
            *Registers = ImgEffectsRegisterMap;
            break;

        case CAMERIC_MODULE_SUPER_IMPOSE:
            *NumRegisters = ARRAY_SIZE( SuperImposeRegisterMap );
            *Registers = SuperImposeRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_MAIN:
            *NumRegisters = ARRAY_SIZE( IspMainRegisterMap );
            *Registers = IspMainRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_FLASH:
            *NumRegisters = ARRAY_SIZE( IspFlashRegisterMap );
            *Registers = IspFlashRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_SHUTTER:
            *NumRegisters = ARRAY_SIZE( IspShutterRegisterMap );
            *Registers = IspShutterRegisterMap;
            break;

        case CAMERIC_MODULE_CPROC:
            *NumRegisters = ARRAY_SIZE( CprocRegisterMap );
            *Registers = CprocRegisterMap;
            break;

        case CAMERIC_MODULE_MRSZ:
            *NumRegisters = ARRAY_SIZE( MrszRegisterMap );
            *Registers = MrszRegisterMap;
            break;

        case CAMERIC_MODULE_SRSZ:
            *NumRegisters = ARRAY_SIZE( SrszRegisterMap );
            *Registers = SrszRegisterMap;
            break;

        case CAMERIC_MODULE_MI:
            *NumRegisters = ARRAY_SIZE( MiRegisterMap );
            *Registers = MiRegisterMap;
            break;

        case CAMERIC_MODULE_JPE:
            *NumRegisters = ARRAY_SIZE( JpeRegisterMap );
            *Registers = JpeRegisterMap;
            break;

        case CAMERIC_MODULE_SMIA:
#if (MARVIN_FEATURE_SMIA == MARVIN_FEATURE_EXIST_NOT)
            *Registers      = NULL;
            *NumRegisters   = 0UL;
#else  /* (MARVIN_FEATURE_SMIA == MARVIN_FEATURE_EXIST_NOT) */
            *NumRegisters = ARRAY_SIZE( SmiaRegisterMap );
            *Registers = SmiaRegisterMap;
#endif /* (MARVIN_FEATURE_SMIA == MARVIN_FEATURE_EXIST_NOT) */
            break;

        case CAMERIC_MODULE_MIPI:
            *NumRegisters = ARRAY_SIZE( MipiRegisterMap );
            *Registers = MipiRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_AFM:
            *NumRegisters = ARRAY_SIZE( AfmRegisterMap );
            *Registers = AfmRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_BP:
            *Registers      = NULL;
            *NumRegisters   = 0UL;
            break;

        case CAMERIC_MODULE_ISP_LSC:
            *NumRegisters = ARRAY_SIZE( LscRegisterMap );
            *Registers = LscRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_IS:
            *NumRegisters = ARRAY_SIZE( IsRegisterMap );
            *Registers = IsRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_HIST:
            *NumRegisters = ARRAY_SIZE( HistRegisterMap );
            *Registers = HistRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_FILTER:
            *NumRegisters = ARRAY_SIZE( FiltRegisterMap );
            *Registers = FiltRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_CAC:
            *NumRegisters = ARRAY_SIZE( CacRegisterMap );
            *Registers = CacRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_CNR:
            *NumRegisters = ARRAY_SIZE( CnrRegisterMap );
            *Registers = CnrRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_EXP:
            *NumRegisters = ARRAY_SIZE( ExpRegisterMap );
            *Registers = ExpRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_BLS:
            *NumRegisters = ARRAY_SIZE( BlsRegisterMap );
            *Registers = BlsRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_DPF:
            *NumRegisters = ARRAY_SIZE( DpfRegisterMap );
            *Registers = DpfRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_DPCC:
            *NumRegisters = ARRAY_SIZE( DpccRegisterMap );
            *Registers = DpccRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_WDR:
            *NumRegisters = ARRAY_SIZE( WdrRegisterMap );
            *Registers = WdrRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_AWB:
            *NumRegisters = ARRAY_SIZE( AwbRegisterMap );
            *Registers = AwbRegisterMap;
            break;

        case CAMERIC_MODULE_ISP_VSM:
            *NumRegisters = ARRAY_SIZE( VsmRegisterMap );
            *Registers = VsmRegisterMap;
            break;

        case CAMERIC_MODULE_INVALID:
        case CAMERIC_MODULE_MAX:
        default:
            *Registers      = NULL;
            *NumRegisters   = 0UL;
            return ( RET_OUTOFRANGE );
    }

    TRACE( CAMERIC_REG_DESCRIPTION_INFO, "%s: (exit)\n", __func__ );

    return ( RET_SUCCESS );
}

